==> GotoBLAS-1.10.tar.gz: GotoBLAS/level3/syrk/syrk_thread.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "common.h"

int syrk_thread(int mode, long n, long k, 
		 double alpha_r, double alpha_i,
		 void *a, long lda,
		 void *b, long ldb,
		 void *c, long ldc, int (*function)(), void *buffer){

  blas_queue_t queue[MAX_CPU_NUMBER];
  long width, i;
  int num_cpu, mask;
  double dnum = (double)n * (double)n /(double)blas_cpu_number;
  
  if (!(mode & BLAS_COMPLEX)) {
    if (!(mode & BLAS_DOUBLE)) mask = SGEMM_UNROLL_M; else mask = DGEMM_UNROLL_M;
  } else {
    if (!(mode & BLAS_DOUBLE)) mask = CGEMM_UNROLL_M; else mask = ZGEMM_UNROLL_M;
  }

  for (i = 0; i < blas_cpu_number; i++) blas_queue_init(&queue[i]);

  num_cpu = 0;
  i = 0;

  while (i < n){
    
    if (blas_cpu_number - num_cpu > 1) {
      if (!(mode & BLAS_UPLO)) {
	double di   = (double)i;

	width = (long)(sqrt(di * di + dnum) - di);
	if (num_cpu == 0) {
	  width = ((width +  mask - 1) & ~(mask - 1));
	} else {
	  width = (width & ~(mask - 1));
	}
      } else {
	double di   = (double)(n - i);
	double dx   = di * di - dnum;

	if (dx > 0) {
	  width = (long)(di - sqrt(dx));
	  if (num_cpu == 0) {
	    width = ((width + mask - 1) & ~(mask - 1));
	  } else {
	    width = (width & ~(mask - 1));
	  }
	} else {
	  width = n - i;
	}
      }

      if ((width > n - i) || (width < mask)) width = n - i;

    } else {
      width = n - i;
    }

    queue[num_cpu].mode = mode;
    queue[num_cpu].routine = function;
    queue[num_cpu].n = n;
    queue[num_cpu].k = k;
    queue[num_cpu].a = (void *)a;
    queue[num_cpu].b = (void *)b;
    queue[num_cpu].c = (void *)c;
    queue[num_cpu].lda = lda;
    queue[num_cpu].ldb = ldb;
    queue[num_cpu].ldc = ldc;
    queue[num_cpu].alpha[0] = alpha_r;
    queue[num_cpu].alpha[1] = alpha_i;
    queue[num_cpu].from = i;
    queue[num_cpu].to   = i + width;
    queue[num_cpu].next  = &queue[num_cpu + 1];

    num_cpu ++;
    i += width;
  }
  
  if (num_cpu) {
    queue[num_cpu - 1].next = NULL;
    queue[num_cpu - 1].last = 1;
    
    exec_blas(queue, buffer);
  }

  for (i = 0; i < blas_cpu_number; i++) blas_queue_destroy(&queue[i]);

  return 0;
}

==> GotoBLAS-1.15.tar.gz: GotoBLAS/level3/syrk/syrk_thread.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "common.h"

int syrk_thread(int mode, long n, long k, 
		 double alpha_r, double alpha_i,
		 void *a, long lda,
		 void *b, long ldb,
		 void *c, long ldc, int (*function)(), void *buffer){

  blas_queue_t queue[MAX_CPU_NUMBER];
  long width, i;
  int num_cpu;
  int mask = 0;
  double dnum = (double)n * (double)n /(double)blas_cpu_number;
  
  if (!(mode & BLAS_COMPLEX)) {

    switch (mode & BLAS_PREC) {
    case BLAS_SINGLE:
      mask = SGEMM_UNROLL_M;
      break;
    case BLAS_DOUBLE:
      mask = DGEMM_UNROLL_M;
      break;
#ifdef EXPRECISION
    case BLAS_XDOUBLE:
      mask = QGEMM_UNROLL_M;
      break;
#endif
    }
  } else {
    switch (mode & BLAS_PREC) {
    case BLAS_SINGLE:
      mask = CGEMM_UNROLL_M;
      break;
    case BLAS_DOUBLE:
      mask = DGEMM_UNROLL_M;
      break;
#ifdef EXPRECISION
    case BLAS_XDOUBLE:
      mask = XGEMM_UNROLL_M;
      break;
#endif
    }
  }

  for (i = 0; i < blas_cpu_number; i++) blas_queue_init(&queue[i]);

  num_cpu = 0;
  i = 0;

  while (i < n){
    
    if (blas_cpu_number - num_cpu > 1) {
      if (!(mode & BLAS_UPLO)) {
	double di   = (double)i;

	width = (long)(sqrt(di * di + dnum) - di);
	if (num_cpu == 0) {
	  width = ((width +  mask - 1) & ~(mask - 1));
	} else {
	  width = (width & ~(mask - 1));
	}
      } else {
	double di   = (double)(n - i);
	double dx   = di * di - dnum;

	if (dx > 0) {
	  width = (long)(di - sqrt(dx));
	  if (num_cpu == 0) {
	    width = ((width + mask - 1) & ~(mask - 1));
	  } else {
	    width = (width & ~(mask - 1));
	  }
	} else {
	  width = n - i;
	}
      }

      if ((width > n - i) || (width < mask)) width = n - i;

    } else {
      width = n - i;
    }

    queue[num_cpu].mode = mode;
    queue[num_cpu].routine = function;
    queue[num_cpu].n = n;
    queue[num_cpu].k = k;
    queue[num_cpu].a = (void *)a;
    queue[num_cpu].b = (void *)b;
    queue[num_cpu].c = (void *)c;
    queue[num_cpu].lda = lda;
    queue[num_cpu].ldb = ldb;
    queue[num_cpu].ldc = ldc;
    queue[num_cpu].alpha[0] = alpha_r;
    queue[num_cpu].alpha[1] = alpha_i;
    queue[num_cpu].from = i;
    queue[num_cpu].to   = i + width;
    queue[num_cpu].next  = &queue[num_cpu + 1];

    num_cpu ++;
    i += width;
  }
  
  if (num_cpu) {
    queue[num_cpu - 1].next = NULL;
    queue[num_cpu - 1].last = 1;
    
    exec_blas(queue, buffer);
  }

  for (i = 0; i < blas_cpu_number; i++) blas_queue_destroy(&queue[i]);

  return 0;
}
